#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#include "assert.h"
#include "aa.h"
#include "vector.h"
#include "pssm.h"

#define INFINITY 10000

/*Log: 5/30/2002
 changed PSSM_scaled_new to let it return scaled_factor
 changed PSSM_print to let it take a scaled_factor
*/

//global variable
//double scale;
//double mymin;
//double mymax;

/************************************************************************
 *   Utilities
 ************************************************************************/

static double
minimum (double x, double y) {
  return (x < y) ? x : y;
}

static double
maximum (double x, double y) {
  return (x > y) ? x : y;
}

void
PSSM_print (Matrix_T pssm, int positions, double scale) {
  extern const int AA_abc_length;
  int i, j;
  Vector_T vector;

  printf("ID\n");
  printf("AC\n");
  printf("DE\n");
  printf("MA\n");
  //add this line for scale factor
  //printf("%3d\n", scale);
  printf("  A   B   C   D   E   F   G   H   I   K   L ");
  printf("  M   N   P   Q   R   S   T   V   W   X   Y   Z   *   -\n");
  for (i = 0; i < positions; ++i) {
    vector = Matrix_get(pssm,i);
    for (j = 0; j < AA_abc_length; ++j) {
      printf("%3d ", (int) Vector_get(vector,j));
    }
    printf("  0   0\n"); 
  }
  printf("//\n");
}

/************************************************************************
*  Make Log-Odds Matrix
*    After this is called, use abc_index rather than acd_index.
************************************************************************/

Matrix_T
PSSM_logodds_new (Matrix_T freqs) {
  extern const int AA_abc_length;
  int indexB, indexD, indexN, indexZ, indexE, indexQ, indexX;
  double partD, partN, partE, partQ;
  double freqD, freqN, freqE, freqQ;
  double p, score, average_score;
  int i, j, positions;
  char aa;
  Matrix_T logodds;
  Vector_T logodds_vector, freqs_vector;

  positions = Matrix_width(freqs);
  logodds = Matrix_new(positions,AA_abc_length);
  indexB = AA_abc_index('B');
  indexD = AA_abc_index('D');
  indexN = AA_abc_index('N');
  indexZ = AA_abc_index('Z');
  indexE = AA_abc_index('E');
  indexQ = AA_abc_index('Q');
  indexX = AA_abc_index('X');

  freqD = AA_acd_freq(indexD);
  freqN = AA_acd_freq(indexN);
  freqE = AA_acd_freq(indexE);
  freqQ = AA_acd_freq(indexQ);

  partD = freqD/(freqD+freqN);
  partN = freqN/(freqD+freqN);
  partE = freqE/(freqE+freqQ);
  partQ = freqQ/(freqE+freqQ);

  for (i = 0; i < positions; ++i) {
    logodds_vector = Vector_new(AA_abc_length);
    freqs_vector = Matrix_get(freqs,i);
    average_score = 0.0;
    for (j = 0; j < NAT_AAS; ++j) {
      p = Vector_get(freqs_vector,j);
      assert(p > 0.0);
      score = log(p/AA_acd_freq(j));
      average_score += score * AA_acd_freq(j);
      aa = AA_acd_aa(j);
      Vector_put(logodds_vector,AA_abc_index(aa),score);
    }
    Vector_put(logodds_vector,indexX,average_score);
    score = partD * Vector_get(logodds_vector,indexD)
      + partN * Vector_get(logodds_vector,indexN);
    Vector_put(logodds_vector,indexB,score);
    score = partE * Vector_get(logodds_vector,indexE)
      + partQ * Vector_get(logodds_vector,indexQ);
    Vector_put(logodds_vector,indexZ,score);
    Matrix_put(logodds,i,logodds_vector);
  }
  return logodds;
}


/************************************************************************
*  Scale Matrix
************************************************************************/

static double
compute_scale_factor (double min, double max) {
  double factor;

  if (max > -min) {
    factor = 99.0 / max;
  } else {
    factor = 99.0 / (-min);
  }
  return factor;
}


Matrix_T
//scaled_PSSM*
PSSM_scaled_new (Matrix_T source) {
  extern const int AA_abc_length;
  int i, j, positions, x;
  double scale_factor, min = INFINITY, max = -INFINITY, score;
  Matrix_T scaled;
  Vector_T scaled_vector, source_vector;
  scaled_PSSM* pssm_ptr;

  positions = Matrix_width(source);
  scaled = Matrix_new(positions,AA_abc_length);
  for (i = 0; i < positions; ++i) {
    source_vector = Matrix_get(source,i);
    for (j = 0; j < AA_abc_length; ++j) {
      score = Vector_get(source_vector,j);
      min = minimum(min,score);
      max = maximum(max,score);
    }
  }
  //testing
  //mymin=min;
  //mymax=max;
  scale_factor = compute_scale_factor(min,max);
  //global variable
  //scale=scale_factor;

  for (i = 0; i < positions; ++i) {
    scaled_vector = Vector_new(AA_abc_length);
    source_vector = Matrix_get(source,i);
    for (j = 0; j < AA_abc_length; ++j) {
      score = Vector_get(source_vector,j);
      x = (int) rint(scale_factor * score);
      Vector_put(scaled_vector,j,x);
       //use this line to print original score for debugging
      //Vector_put(scaled_vector,j,score);
      
    }
    Matrix_put(scaled,i,scaled_vector);
  }
  pssm_ptr=(scaled_PSSM*)malloc(sizeof(scaled_PSSM));
  pssm_ptr->pssm=&scaled;
  pssm_ptr->scale=scale_factor;
  return pssm_ptr;
  //return scaled;
}

